@@ -1,4 +1,11 @@
+# We'll tell isort not to sort these base classes
+# isort: off
+
 from .basis import Basis
+from .steerable import SteerableBasis2D
+
+# isort: on
+
 from .dirac import DiracBasis
 from .fb_2d import FBBasis2D
 from .fb_3d import FBBasis3D
@@ -6,6 +13,7 @@ from .ffb_2d import FFBBasis2D
 from .ffb_3d import FFBBasis3D
 from .fpswf_2d import FPSWFBasis2D
 from .fpswf_3d import FPSWFBasis3D
+from .fspca import FSPCABasis
 from .polar_2d import PolarBasis2D
 from .pswf_2d import PSWFBasis2D
 from .pswf_3d import PSWFBasis3D
