@@ -175,7 +175,10 @@ class Image:
         return Image(-self.data)
 
     def sqrt(self):
-        return np.sqrt(self.data)
+        return Image(np.sqrt(self.data))
+
+    def flip_axes(self):
+        return Image(np.transpose(self.data, (0, 2, 1)))
 
     def __repr__(self):
         return f"{self.n_images} images of size {self.res}x{self.res}"
@@ -357,6 +360,9 @@ class Image:
         :param figsize: Figure size in inches, consult `matplotlib.figure`.
         """
 
+        # We never need more columns than images.
+        columns = min(columns, self.n_images)
+
         plt.figure(figsize=figsize)
         for i, im in enumerate(self):
             plt.subplot(self.n_images // columns + 1, columns, i + 1)
