@@ -13,4 +13,6 @@ from .filters import (
     ScalarFilter,
     ScaledFilter,
     ZeroFilter,
+    voltage_to_wavelength,
 )
+from .wemd import wemd_embed, wemd_norm
