@@ -30,6 +30,12 @@ class Estimator:
                 f" basis: {self.basis.dtype}"
             )
 
+        if src.L != basis.nres:
+            raise ValueError(
+                "Currently require 2D source and 3D volume resolution to be the same."
+                f" Given src.L={src.L} != {basis.nres}"
+            )
+
         """
         An object representing a 2*L-by-2*L-by-2*L array containing the non-centered Fourier transform of the mean
         least-squares estimator kernel.
@@ -63,7 +69,7 @@ class Estimator:
         raise NotImplementedError("Subclasses must implement the compute_kernel method")
 
     def estimate(self, b_coeff=None, tol=None):
-        """ Return an estimate as a Volume instance. """
+        """Return an estimate as a Volume instance."""
         if b_coeff is None:
             b_coeff = self.src_backward()
         est_coeff = self.conj_grad(b_coeff, tol=tol)
