@@ -1,5 +1,6 @@
 import logging
 import os.path
+from collections import OrderedDict
 
 import mrcfile
 import numpy as np
@@ -15,8 +16,13 @@ from aspire.image.xform import (
     Multiply,
     Pipeline,
 )
-from aspire.operators import LambdaFilter, MultiplicativeFilter, PowerFilter
-from aspire.storage import MrcStats, StarFile, StarFileBlock
+from aspire.operators import (
+    IdentityFilter,
+    LambdaFilter,
+    MultiplicativeFilter,
+    PowerFilter,
+)
+from aspire.storage import MrcStats, StarFile
 from aspire.utils import ensure
 from aspire.utils.coor_trans import grid_2d
 
@@ -100,6 +106,12 @@ class ImageSource:
         # The private attribute '_cached_im' can be populated by calling this object's cache() method explicitly
         self._cached_im = None
 
+        # _rotations is assigned non None value
+        #  by `rots` or `angles` setters.
+        #  It is potentially used by sublasses to test if we've used setters.
+        #  This must come before the Relion/starfile meta data parsing below.
+        self._rotations = None
+
         if metadata is None:
             self._metadata = pd.DataFrame([], index=pd.RangeIndex(self.n))
         else:
@@ -119,7 +131,7 @@ class ImageSource:
 
     @property
     def states(self):
-        return self.get_metadata("_rlnClassNumber")
+        return np.atleast_1d(self.get_metadata("_rlnClassNumber"))
 
     @states.setter
     def states(self, values):
@@ -164,7 +176,9 @@ class ImageSource:
 
     @property
     def offsets(self):
-        return self.get_metadata(["_rlnOriginX", "_rlnOriginY"], default_value=0.0)
+        return np.atleast_2d(
+            self.get_metadata(["_rlnOriginX", "_rlnOriginY"], default_value=0.0)
+        )
 
     @offsets.setter
     def offsets(self, values):
@@ -172,7 +186,7 @@ class ImageSource:
 
     @property
     def amplitudes(self):
-        return self.get_metadata("_rlnAmplitude", default_value=1.0)
+        return np.atleast_1d(self.get_metadata("_rlnAmplitude", default_value=1.0))
 
     @amplitudes.setter
     def amplitudes(self, values):
@@ -183,13 +197,28 @@ class ImageSource:
         """
         :return: Rotation angles in radians, as a n x 3 array
         """
-        return self._rotations.as_euler().astype(self.dtype)
+        # Call a private method. This allows sub classes to effeciently override.
+        return self._angles()
+
+    def _angles(self):
+        """
+        Converts internal _rotations representation to expected matrix form.
+        """
+        return self._rotations.as_euler("ZYZ", degrees=False).astype(self.dtype)
 
     @property
     def rots(self):
         """
         :return: Rotation matrices as a n x 3 x 3 array
         """
+        # Call a private method. This allows sub classes to effeciently override.
+        return self._rots()
+
+    def _rots(self):
+        """
+        Converts internal `_rotations` representation to expected matrix form.
+        :return: Rotation matrices as a n x 3 x 3 array
+        """
         return self._rotations.as_matrix().astype(self.dtype)
 
     @angles.setter
@@ -361,7 +390,7 @@ class ImageSource:
         :param kwargs: Any additional keyword arguments to pass on to the `ImageSource`'s underlying `_images` method.
         :return: an `Image` object.
         """
-        indices = np.arange(start, min(start + num, self.n), dtype=np.int)
+        indices = np.arange(start, min(start + num, self.n), dtype=int)
 
         if self._cached_im is not None:
             logger.info("Loading images from cache")
@@ -590,50 +619,50 @@ class ImageSource:
             axis=1,
         )
 
-        filename_indices = None
+        if new_mrcs:
+            # Create a new column that we will be populating in the loop below
+            df["_rlnImageName"] = ""
 
-        with open(starfile_filepath, "w") as f:
-            if new_mrcs:
-                # Create a new column that we will be populating in the loop below
-                # For
-                df["_rlnImageName"] = ""
+            if save_mode == "single":
+                # Save all images into one single mrc file
+                fname = os.path.basename(starfile_filepath)
+                fstem = os.path.splitext(fname)[0]
+                mrcs_filename = f"{fstem}_{0}_{self.n-1}.mrcs"
 
-                if save_mode == "single":
-                    # Save all images into one single mrc file
-                    fname = os.path.basename(starfile_filepath)
-                    fstem = os.path.splitext(fname)[0]
-                    mrcs_filename = f"{fstem}_{0}_{self.n-1}.mrcs"
-
-                    # Then set name in dataframe for the StarFile
-                    df["_rlnImageName"][0 : self.n] = pd.Series(
-                        [f"{j + 1:06}@{mrcs_filename}" for j in range(self.n)]
+                # Then set name in dataframe for the StarFile
+                # Note, here the row_indexer is :, representing all rows in this data frame.
+                #   df.loc will be reponsible for dereferencing and assigning values to df.
+                #   Pandas will assert df.shape[0] == self.n
+                df.loc[:, "_rlnImageName"] = [
+                    f"{j + 1:06}@{mrcs_filename}" for j in range(self.n)
+                ]
+            else:
+                # save all images into multiple mrc files in batch size
+                for i_start in np.arange(0, self.n, batch_size):
+                    i_end = min(self.n, i_start + batch_size)
+                    num = i_end - i_start
+                    mrcs_filename = (
+                        os.path.splitext(os.path.basename(starfile_filepath))[0]
+                        + f"_{i_start}_{i_end-1}.mrcs"
                     )
+                    # Note, here the row_indexer is a slice.
+                    #   df.loc will be reponsible for dereferencing and assigning values to df.
+                    #   Pandas will assert the lnegth of row_indexer equals num.
+                    row_indexer = df[i_start:i_end].index
+                    df.loc[row_indexer, "_rlnImageName"] = [
+                        "{0:06}@{1}".format(j + 1, mrcs_filename) for j in range(num)
+                    ]
 
-                else:
-                    # save all images into multiple mrc files in batch size
-                    for i_start in np.arange(0, self.n, batch_size):
-                        i_end = min(self.n, i_start + batch_size)
-                        num = i_end - i_start
-                        mrcs_filename = (
-                            os.path.splitext(os.path.basename(starfile_filepath))[0]
-                            + f"_{i_start}_{i_end-1}.mrcs"
-                        )
-
-                        df["_rlnImageName"][i_start:i_end] = pd.Series(
-                            [
-                                "{0:06}@{1}".format(j + 1, mrcs_filename)
-                                for j in range(num)
-                            ]
-                        )
-
-            filename_indices = [
-                df["_rlnImageName"][i].split("@")[1] for i in range(self.n)
-            ]
-
-            # initial the star file object and save it
-            starfile = StarFile(blocks=[StarFileBlock(loops=[df])])
-            starfile.save(f)
+        filename_indices = df._rlnImageName.str.split(pat="@", expand=True)[1].tolist()
 
+        # initialize the star file object and save it
+        odict = OrderedDict()
+        # since our StarFile only has one block, the convention is to save it with the header "data_", i.e. its name is blank
+        # if we had a block called "XYZ" it would be saved as "XYZ"
+        # thus we index the metadata block with ""
+        odict[""] = df
+        out_star = StarFile(blocks=odict)
+        out_star.write(starfile_filepath)
         return filename_indices
 
     def save_images(
@@ -732,13 +761,70 @@ class ArrayImageSource(ImageSource):
     if available, is consulted directly by the parent class, bypassing `_images`.
     """
 
-    def __init__(self, im, metadata=None):
+    def __init__(self, im, metadata=None, angles=None):
         """
         Initialize from an `Image` object
-        :param im: An `Image` object representing image data served up by this `ImageSource`
+        :param im: An `Image` or Numpy array object representing image data served up by this `ImageSource`.
+        In the case of a Numpy array, attempts to create an 'Image' object.
         :param metadata: A Dataframe of metadata information corresponding to this ImageSource's images
+        :param angles: Optional n-by-3 array of rotation angles corresponding to `im`.
         """
+
+        if not isinstance(im, Image):
+            logger.info("Attempting to create an Image object from Numpy array.")
+            try:
+                im = Image(im)
+            except Exception as e:
+                raise RuntimeError(
+                    "Creating Image object from Numpy array failed."
+                    f" Original error: {str(e)}"
+                )
+
         super().__init__(
             L=im.res, n=im.n_images, dtype=im.dtype, metadata=metadata, memory=None
         )
+
         self._cached_im = im
+
+        # Create filter indices, these are required to pass unharmed through filter eval code
+        #   that is potentially called by other methods later.
+        self.filter_indices = np.zeros(self.n)
+        self.unique_filters = [IdentityFilter()]
+
+        # Optionally populate angles/rotations.
+        if angles is not None:
+            if angles.shape != (self.n, 3):
+                raise ValueError(f"Angles should be shape {(self.n, 3)}")
+            # This will populate `_rotations`,
+            #   which is exposed by properties `angles` and `rots`.
+            self.angles = angles
+
+    def _rots(self):
+        """
+        Private method, checks if `_rotations` has been set,
+        then returns inherited rots, otherwise raise.
+        """
+
+        if self._rotations is not None:
+            return super()._rots()
+        else:
+            raise RuntimeError(
+                "Consumer of ArrayImageSource trying to access rots,"
+                " but rots were not defined for this source."
+                "  Try instantiating with angles."
+            )
+
+    def _angles(self):
+        """
+        Private method, checks if `_rotations` has been set,
+        then returns inherited angles, otherwise raise.
+        """
+
+        if self._rotations is not None:
+            return super()._angles()
+        else:
+            raise RuntimeError(
+                "Consumer of ArrayImageSource trying to access angles,"
+                " but angles were not defined for this source."
+                "  Try instantiating with angles."
+            )
